SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Services'!
SystemOrganization addCategory: #'Rest-Server'!

Object subclass: #RestGetRequestProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestGetRequestProcessor methodsFor: 'converting' stamp: 'mt 2/23/2009 11:50'!
convertObject: anObject
	"Create a dictionary with properties for the given object."
	
	^ anObject class allInstVarNames
		inject: Dictionary new
		into: [:old :varName| old at: varName put: (anObject instVarNamed: varName); yourself].! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'mt 2/23/2009 11:34'!
process: aRoot components: someComponents
	| result |
	result := self resolveId: aRoot all components: someComponents.

	result ifKindOf: HTTPResponse thenDo: [^ result].
	^ HTTPResponse ok entity: (Json render:
			(result isCollection
			ifTrue: [result]
			ifFalse: [self convertObject: result])).! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'mt 2/23/2009 11:04'!
resolveId: aCollection components: pathComponents
	| id resource |
	pathComponents ifEmpty: [^ aCollection].
	
	id := pathComponents first asNumber.
	resource := aCollection 
		detect: [:each | each id = id] 
		ifNone: [RestResolveError signal: ('Entity with id {1} not found.' format: {id})].

	^ self resolveMethod: resource components: pathComponents allButFirst! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'mt 2/23/2009 11:06'!
resolveMethod: anEntity components: pathComponents
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := pathComponents first asSymbol.
	(anEntity class allSelectors includes: selector)
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].

	^ self resolveId: resource components: pathComponents allButFirst! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler class methodsFor: 'error handling' stamp: 'mfp 2/20/2009 15:37'!
responseForFailure
	^ HTTPResponse internalServerError.! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'mt 2/23/2009 12:21'!
process: anHTTPRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."

	| pathComponents root |
	pathComponents := anHTTPRequest uri identifierPath allButFirst. "remove leading '/'"
	root := self resources
		at: pathComponents first
		ifAbsent: [^ self class responseForFailure].
	[anHTTPRequest isGet ifTrue:
		[^ RestGetRequestProcessor new process: root components: pathComponents allButFirst].
	"anHTTPRequest isPost ifTrue:
		[^ self processPost: res content: anHTTPRequest ensureFullRead entityBody value].
	anHTTPRequest isPut ifTrue:
		[^ self processPut: res content: anHTTPRequest putFields].
	anHTTPRequest isDelete ifTrue:
		[^ self processDelete: res]."]
		on: RestResolveError do: [:e | ^ e asHTTPResponse]! !

!RestResourceHttpHandler methodsFor: 'constants' stamp: 'mt 2/23/2009 12:20'!
resources
	
	self subclassResponsibility! !

!HttpRequest methodsFor: '*rest-server' stamp: 'mt 2/18/2009 14:21'!
putFields
	"Answer the multipart fields for a post request (if in fact this is a
	PUT request"

	^self propertyAt: #putFields ifAbsentPut: [
		" parse PUT fields in the body of request, if it's standard form data "

		(self isPutRequest and: [ 
			self contentType = MIMEDocument contentTypeFormData 
		]) ifTrue: [
			self class
				decodeUrlEncodedForm: (stream next: self contentLength)
				multipleValues: true.
		] ifFalse: [
			Dictionary new
		].
	]! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:05'!
isRunning
	^site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 2/19/2009 17:51'!
port
	^site port! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 17:15'!
service
	^resource! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:08'!
start

	self startOn: 8082.

! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 2/19/2009 17:59'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^self].
	site := Site new
		name: 'rest';
		host: 'localhost' ip: '127.0.0.1' port: aPort;
		yourself.

	SwazooServer singleton addSite: site.
	
	resource := RestHttpService new uriPattern: '/'.
	site
		"start;"
		addResource: resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 17:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	site ifNotNil:
		[site stop.
		SwazooServer singleton removeSite: site.
		site := nil.
		resource := nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/19/2009 17:49'!
answerTo: request
	
	^ (handlers
		detect: [:ass| request uri asString startsWith: ass key]
		ifNone: [^ HTTPResponse notFound])
			value new process: request! !

!RestHttpService methodsFor: 'initialization' stamp: 'mt 2/2/2009 10:12'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/15/2009 16:07'!
installHandler: aRequestHandlerClass on: aPathString
	(handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass! !

!Collection methodsFor: '*rest-server' stamp: 'cs 2/16/2009 00:38'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

Error subclass: #RestResolveError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResolveError methodsFor: 'as yet unclassified' stamp: 'mt 2/23/2009 11:48'!
asHTTPResponse
	^ HTTPResponse notFound
		entity: self messageText! !
